@@ -392,6 +392,37 @@ class DataFrame(metaclass=PureStaticDataFrameMethods):
         df._index = self._index
         return df
 
+    def astype(
+        self,
+        dtype: np.dtype | str | type | Mapping[Hashable, np.dtype | str | type],
+        copy: bool = True,
+        errors: Literal["raise", "ignore"] = "raise",
+    ) -> "DataFrame":
+        """
+        Convert the data type of columns.
+
+        :param dtype: Global data type for all the columns or mapping from column keys to the new \
+                      data types.
+        :param copy: Value indicating whether the operation copies the original DataFrame.
+        :param errors: Either "raise" (default) or "ignore". If set to "ignore", silently skip \
+                       columns which failed dtype conversion.
+        :return: Converted DataFrame.
+        """
+        if not isinstance(dtype, Mapping):
+            dtype = {k: dtype for k in self._columns}
+
+        df = self.copy() if copy else self
+
+        columns = df._columns
+        for k, k_dt in dtype.items():
+            try:
+                columns[k] = columns[k].astype(k_dt, copy=False)
+            except ValueError as e:
+                if errors == "raise":
+                    raise e from None
+
+        return df
+
     @property
     def index(self) -> Index:
         """Return the associated index."""
@@ -434,9 +465,9 @@ class DataFrame(metaclass=PureStaticDataFrameMethods):
         elif len(by) == 0:
             raise ValueError("must specify at least one column")
 
-        df = self.copy() if not inplace else self
+        df = self if inplace else self.copy()
 
-        if not ignore_index and df._index == ():
+        if not ignore_index and len(df._index) == 0:
             df["_index0"] = np.arange(len(self), dtype=int)
             df._index = ("_index0",)
 
@@ -512,13 +543,13 @@ class DataFrame(metaclass=PureStaticDataFrameMethods):
                         returning a copy.
         :param drop: Erase the old indexed columns, except those mentioned in the new index.
         """
-        df = self.copy() if not inplace else self
+        df = self if inplace else self.copy()
         old_index = df._index
 
         if isinstance(index, list) or (isinstance(index, np.ndarray) and index.ndim > 1):
             index = tuple(index)
 
-        if index == ():
+        if isinstance(index, tuple) and len(index) == 0:
             df._index = index
         elif isinstance(index, np.ndarray):
             if "_index0" in df and not drop:
@@ -564,7 +595,7 @@ class DataFrame(metaclass=PureStaticDataFrameMethods):
                         returning a copy.
         :param drop: Delete the columns referenced by `index`.
         """
-        df = self.copy() if not inplace else self
+        df = self if inplace else self.copy()
 
         if df._index and drop:
             for c in df._index:
@@ -573,6 +604,40 @@ class DataFrame(metaclass=PureStaticDataFrameMethods):
         df._index = ()
         return df
 
+    def rename(
+        self,
+        columns: Mapping[Hashable, Hashable],
+        inplace: bool = False,
+        errors: Literal["raise", "ignore"] = "ignore",
+    ) -> "DataFrame":
+        """
+        Rename columns in bulk.
+
+        :param columns: Mapping from old to new column keys.
+        :param inplace: Value indicating whether we must update the current DataFrame instead of \
+                        returning a copy.
+        :param errors: "ignore" suppresses missing columns, "raise" forwards KeyError.
+        :return: DataFrame with renamed columns.
+        """
+        df = self if inplace else self.copy()
+        if not isinstance(columns, Mapping):
+            raise TypeError(f"columns must be a mapping, got {type(columns)}")
+        new_columns = {}
+        for old, new in columns.items():
+            try:
+                new_columns[new] = df._columns[old]
+            except KeyError as e:
+                if errors == "raise":
+                    raise e from None
+                continue
+            del df._columns[old]
+        for key, val in new_columns.items():
+            df[key] = val
+
+        df._index = tuple(columns.get(i, i) for i in df._index)
+
+        return df
+
     def drop_duplicates(
         self,
         subset: Iterable[Hashable],
@@ -672,8 +737,10 @@ class DataFrame(metaclass=PureStaticDataFrameMethods):
         """
         Return the unique elements of `column`.
 
+        :param column: Column key to select.
         :param unordered: Value indicating whether the result must be sorted or the caller \
                           doesn't care.
+        :return: Unique column values.
         """
         values = self[column]
         if unordered:
@@ -741,6 +808,32 @@ class DataFrame(metaclass=PureStaticDataFrameMethods):
             return None
         return m
 
+    def fillna(
+        self,
+        value: Any,
+        column: Hashable | None = None,
+        inplace: bool = False,
+    ) -> "DataFrame":
+        """
+        Set null-like elements (None, NaN, NaT) to `value`.
+
+        :param value: Scalar replacement value for null-likes.
+        :param column: Affected column key. The default is all columns.
+        :param inplace: Update the current DataFrame instead of returning a copy.
+        :return: Filled DataFrame.
+        """
+        df = self if inplace else self.copy()
+        filled: Iterable[Hashable]
+        if column is None:
+            filled = df._columns
+        else:
+            filled = (column,)
+        columns = df._columns
+        for c in filled:
+            v = columns[c]
+            v[df.isnull(c)] = value
+        return df
+
     @property
     def iloc(self) -> Iloc:
         """Get row by absolute index."""
